#!BPY

"""
Name: 'plot motion'
Blender: 249
Group: 'Armature'
Tooltip: 'plot motion.'
"""

__author__ = "Kuroda Dycoon"
__url__ = ['http://d.hatena.ne.jp/Dycoon/', 'http://d.hatena.ne.jp/Dycoon/']
__version__ = "0.01"

__doc__ = """\
plot motion.
This file is PUBLIC DOMAIN.
"""

import Blender
from Blender import Armature, Mesh, Scene, Window, sys, Image, Draw
from Blender.Mathutils import *
import BPyMesh
import BPyArmature
import BPyObject
import BPySys
import BPyMessages
from BPyArmature import getBakedPoseData
import json
from Blender import *

animationData = {}
boneIndex = {}

def getChildren(obj):	
	obs = Blender.Scene.GetCurrent().objects
	return [ ob for ob in obs if ob.parent == obj ]

def constructObjectAnimation(ob, currentFrame):
	
	global animationData
	
	if not (ob.name in animationData):
		animationData[ob.name] = []
	
	matrix = ob.getMatrix('localspace').copy()
	scl = matrix.scalePart()
	quat = matrix.toQuat()
	loc = matrix.translationPart()
	
	animationData[ob.name].append([currentFrame, [[scl.x, scl.y, scl.z], [quat.x, quat.y, quat.z, quat.w], [loc.x, loc.y, loc.z]]])

def constructArmatureAnimation(ob, ob_data, currentFrame):
	
	global animationData
	global boneIndex
	
	pose = ob.getPose()
	armature_data= ob.getData();
	pose_bones = pose.bones
	
	armature_bone_data = []
	bones_index = {}
	for bone_name, rest_bone in armature_data.bones.items():
		if not (ob.name + "." + bone_name in animationData):
			animationData[ob.name + "." + bone_name] = []
		pose_bone = pose_bones[bone_name]
		rest_matrix = rest_bone.matrix['ARMATURESPACE']
		rest_matrix_inv = rest_matrix.copy().invert()
		armature_bone_data.append( [len(bones_index), -1, bone_name, rest_bone, rest_matrix, rest_matrix_inv, pose_bone, None ])
		bones_index[bone_name] = len(bones_index)
	
	for bone_name, pose_bone in pose_bones.items():
		parent = pose_bone.parent
		if parent:
			bone_index = bones_index[bone_name]
			parent_index = bones_index[parent.name]
			armature_bone_data[ bone_index ][1] = parent_index

		
	for index, parent_index, bone_name, rest_bone, rest_matrix, rest_matrix_inv, pose_bone, ipo in armature_bone_data:
		matrix = pose_bone.poseMatrix
		parent_bone = rest_bone.parent
		if parent_index != -1:
			parent_pose_matrix = armature_bone_data[parent_index][6].poseMatrix
			parent_bone_matrix_inv = armature_bone_data[parent_index][5]
			matrix = matrix * parent_pose_matrix.copy().invert()
			rest_matrix = rest_matrix * parent_bone_matrix_inv
		
		matrix = matrix * rest_matrix.copy().invert()
		scl = matrix.scalePart()
		quat = matrix.toQuat()
		loc = matrix.translationPart()
		
		animationData[ob.name + "." + bone_name].append([currentFrame, [[scl.x, scl.y, scl.z], [quat.x, quat.y, quat.z, quat.w], [loc.x, loc.y, loc.z]]])
		boneIndex[ob.name + "." + bone_name] = index
	
	return

def constructAnimation(objects, currentFrame):
	
	for ob_main in objects:
		
		ob = ob_main
		
		constructObjectAnimation(ob, currentFrame)
		
		ob_data = ob.getData()
		
		if type(ob_data) == Blender.Types.ArmatureType:
			constructArmatureAnimation(ob, ob_data, currentFrame)

		if getChildren(ob):
			constructAnimation(getChildren(ob), currentFrame)
	

def removeNoMotion():
	
	global animationData
	
	for iKey, iVal in animationData.items():
		
		if len(iVal) > 0:
			
			base = iVal[0]
			needDelete = True
			
			for j in iVal:
				if base[1][0][0] != j[1][0][0] or base[1][0][1] != j[1][0][1] or base[1][0][2] != j[1][0][2] or\
						base[1][1][0] != j[1][1][0] or base[1][1][1] != j[1][1][1] or base[1][1][2] != j[1][1][2] or base[1][1][3] != j[1][1][3] or\
						base[1][2][0] != j[1][2][0] or base[1][2][1] != j[1][2][1] or base[1][2][2] != j[1][2][2]:
					needDelete = False
				
			if needDelete:
				animationData[iKey] = [];
				
	

def setObjectAnimation(ob, currentFrame, startFrame):
	
	global animationData
	
	if not (ob.name in animationData):
		return
	
	if len(animationData[ob.name]) > 0:
		ob.insertIpoKey(Blender.Object.IpoKeyTypes.LOCROTSIZE)
		#print("ob.name " + ob.name)
	

def setArmatureAnimation(ob, ob_data, currentFrame, startFrame):
	
	global animationData
	global boneIndex
	
	pose = ob.getPose()
	armature_data= ob.getData();
	pose_bones = pose.bones
	
	for bone_name, pose_bone in pose_bones.items():
		
		#print("ob.name->bone_name " + ob.name + "." + bone_name)
		
		if (ob.name + "." + bone_name) in animationData:
			if len(animationData[ob.name + "." + bone_name]) > 0 :
				
				pose_bone.size.x = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][0][0];
				pose_bone.size.y = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][0][1];
				pose_bone.size.z = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][0][2];
				
				pose_bone.quat.x = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][1][0];
				pose_bone.quat.y = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][1][1];
				pose_bone.quat.z = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][1][2];
				pose_bone.quat.w = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][1][3];
				
				pose_bone.loc.x = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][2][0];
				pose_bone.loc.y = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][2][1];
				pose_bone.loc.z = animationData[ob.name + "." + bone_name][currentFrame - startFrame][1][2][2];
				
				pose_bone.insertKey(ob, currentFrame, Object.Pose.LOC)
				pose_bone.insertKey(ob, currentFrame, Object.Pose.ROT)
				pose_bone.insertKey(ob, currentFrame, Object.Pose.SIZE)
				

def setAnimation(objects, currentFrame, startFrame):
	
	for ob_main in objects:
		
		ob = ob_main
		
		setObjectAnimation(ob, currentFrame, startFrame)
		
		ob_data = ob.getData()
		
		if type(ob_data) == Blender.Types.ArmatureType:
			setArmatureAnimation(ob, ob_data, currentFrame, startFrame)

		if getChildren(ob):
			setAnimation(getChildren(ob), currentFrame, startFrame)
	


def main():
	global animationData

	orig_scene = Scene.GetCurrent()
	export_scenes = Scene.Get()
	
	Window.EditMode(0)
	Window.WaitCursor(1)
	
	for scn in export_scenes:
		scn.makeCurrent()
		context = scn.getRenderingContext()
		orig_frame = Blender.Get('curframe')
		
		animationData = {}
		
		scene_frames = xrange(context.startFrame(), context.endFrame() + 1)
		
		timeLine = scn.getTimeLine()
		
		vf = context.endFrame() - context.startFrame()
		start = sys.time()
		for frame in scene_frames:
			if frame == 15: print vf * (sys.time() - start),"seconds estimated..."
			elif frame > 15:
				percom = frame * 100 / vf
				print "Frame %i Overall %i percent complete\r" % (frame, percom),
			
			Blender.Set('curframe', frame)
			
			constructAnimation(scn.objects, frame)
	
	removeNoMotion();
	
	for scn in export_scenes:
		scn.makeCurrent()
		context = scn.getRenderingContext()
		orig_frame = Blender.Get('curframe')
		
		scene_frames = xrange(context.startFrame(), context.endFrame() + 1)
		
		timeLine = scn.getTimeLine()
		
		vf = context.endFrame() - context.startFrame()
		start = sys.time()
		for frame in scene_frames:
			if frame == 15: print vf * (sys.time() - start),"seconds estimated..."
			elif frame > 15:
				percom = frame * 100 / vf
				print "Frame %i Overall %i percent complete\r" % (frame, percom),
			
			Blender.Set('curframe', frame)
			
			setAnimation(scn.objects, frame, context.startFrame())
	
	for iKey, iVal in animationData.items():
		print("iKey " + iKey + " %d" % (len(iVal)));
		








if __name__=='__main__':
	main()
